/*
** Graphical User Interface using Be API
** for Joysticks ACEpansion
*/

#include <Box.h>
#include <CheckBox.h>
#include <GroupView.h>
#include <LayoutBuilder.h>
#include <Locale.h>
#include <OptionPopUp.h>
#include <StringView.h>

#include <stdio.h>

#include <acepansion/lib_header.h>

#include "acepansion.h"
#include "interface.h"
#define CATCOMP_NUMBERS
#include <generated/locale_strings.h>

#define MUIA_Selected 'MSEL'
#define MUIA_CycleActive 'MCYA'

static BOptionPopUp* makeJoystickObject();
static BView* makeAnalogCheckmarkObject(BCheckBox**);

class JoystickView: public BGroupView
{
	public:
	JoystickView(struct ACEpansionPlugin* myPlugin);

	void AttachedToWindow() override;
	void MessageReceived(BMessage*) override;

	public:
	BCheckBox* CHK_Combined;

	struct ACEpansionPlugin* fPlugin;
	BCheckBox* CHK_Joy0;
	BCheckBox* CHK_Joy1;

    BOptionPopUp *CYA_Select0;
    BOptionPopUp *CYA_Select1;
};

JoystickView::JoystickView(struct ACEpansionPlugin* myPlugin)
	: BGroupView(B_VERTICAL)
	, fPlugin(myPlugin)
{
		BBox* box1 = new BBox("assign");
		box1->SetLabel(GetString(MSG_GROUP_ASSIGN_TITLE));

		BGridView* grid1 = new BGridView();
		box1->AddChild(grid1);

		BLayoutBuilder::Grid<>(grid1)
			.SetInsets(B_USE_SMALL_SPACING)
			.Add(new BStringView("label 0", GetString(MSG_LABEL_JOY0)), 0, 0)
			.Add(CYA_Select0 = makeJoystickObject(), 1, 0)
			.Add(makeAnalogCheckmarkObject(&CHK_Joy0), 1, 1)

			.Add(new BStringView("label 0", GetString(MSG_LABEL_JOY1)), 0, 2)
			.Add(CYA_Select1 = makeJoystickObject(), 1, 2)
			.Add(makeAnalogCheckmarkObject(&CHK_Joy1), 1, 3)
		.End();

		BBox* box2 = new BBox("advanced");
		box2->SetLabel(GetString(MSG_GROUP_ADVANCED_TITLE));

		CHK_Combined = new BCheckBox(GetString(MSG_LABEL_CHK_COMBINED), new BMessage(MUIA_Selected));
		BGroupView* group2 = new BGroupView(B_HORIZONTAL);
		BLayoutBuilder::Group<>(group2)
			.SetInsets(B_USE_SMALL_SPACING)
			.Add(CHK_Combined)
			.AddGlue()
		.End();
		box2->AddChild(group2);

		BLayoutBuilder::Group<>(this)
			.Add(box1)
			.Add(box2)
		.End();
}

void
JoystickView::AttachedToWindow()
{
	CYA_Select0->SetTarget(this);
	CYA_Select1->SetTarget(this);
	CHK_Combined->SetTarget(this);
	CHK_Joy0->SetTarget(this);
	CHK_Joy1->SetTarget(this);
}

void
JoystickView::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case MUIA_CycleActive:
		{
			void* source = NULL;
			message->FindPointer("source", &source);
			int32 idx = message->FindInt32("be:value");
			if (source == CYA_Select0)
				Plugin_SetJoystick(fPlugin, 0, idx);
			else if (source == CYA_Select1)
				Plugin_SetJoystick(fPlugin, 1, idx);
			else
				message->PrintToStream();
			break;
		}
		case 'ANLG':
		{
			void* source = NULL;
			message->FindPointer("source", &source);
			int32 analogMode = message->FindInt32("be:value");
			if (source == CHK_Joy0) {
				Plugin_SetJoystickAnalogMode(fPlugin, 0, analogMode);
			} else if (source == CHK_Joy1)
				Plugin_SetJoystickAnalogMode(fPlugin, 1, analogMode);
			else
				message->PrintToStream();
			break;
		}
		case MUIA_Selected:
		{
            // Notify combined mode
			int32 idx = message->FindInt32("be:value");
			Plugin_SetJoystickCombinedMode(fPlugin, idx);
			CYA_Select1->SetEnabled(idx == 0);
			CHK_Joy1->SetEnabled(idx == 0);
			break;
		}
		default:
			BGroupView::MessageReceived(message);
	}
}


static BOptionPopUp* makeJoystickObject()
{
	return new(std::nothrow) BOptionPopUp("", "", new(std::nothrow) BMessage(MUIA_CycleActive));
}

static BView* makeAnalogCheckmarkObject(BCheckBox** storage)
{
	BGroupView* obj = new BGroupView(B_HORIZONTAL);

	BLayoutBuilder::Group<>(obj)
		.Add(*storage = new BCheckBox(GetString(MSG_CHK_ANALOG), new BMessage('ANLG')))
		.Add(new BStringView("", GetString(MSG_AMSTRAD_PLUS_ONLY)));

	return obj;
}

/*
** Function which is called to initialize commons just
** after the library was loaded into memory and prior
** to any other API call
**
** This is the good place to open our required libraries
** and create our MUI custom classes
*/
BOOL GUI_InitResources(VOID)
{
	return TRUE;
}

/*
** Function which is called to free commons just before
** the library is expurged from memory
**
** This is the good place to close our required libraries
** and destroy our MUI custom classes
*/
void GUI_FreeResources(void)
{
}

/*
** Function which is called from CreatePlugin() to build the GUI
*/
Object * GUI_Create(struct ACEpansionPlugin *plugin)
{
	return (Object*)new(std::nothrow) JoystickView(plugin);
}

/*
** Function which is called from DeletePlugin() to delete the GUI
*/
VOID GUI_Delete(Object *gui)
{
	BView* view = (BView*)gui;
	BWindow* window = view->Window();
	window->Lock();
	view->RemoveSelf();
	window->Unlock();
	delete view;
}

/*
** Functions called from the plugin
*/
void GUI_SetJoystickList(Object *gui, STRPTR*list, ULONG selectedId0, ULONG selectedId1)
{
	JoystickView* v = (JoystickView*)gui;
	v->LockLooper();
	int k = v->CYA_Select0->CountOptions();
	while (k > 0) {
		v->CYA_Select0->RemoveOptionAt(0);
		k--;
	}

	k = v->CYA_Select1->CountOptions();
	while (k > 0) {
		v->CYA_Select1->RemoveOptionAt(0);
		k--;
	}

	for (k = 0; list[k]; k++) {
		v->CYA_Select0->AddOption(list[k], k);
		v->CYA_Select1->AddOption(list[k], k);
	}

	v->CYA_Select0->SelectOptionFor(selectedId0);
	v->CYA_Select1->SelectOptionFor(selectedId1);
	v->UnlockLooper();
}

void GUI_SetJoystickCombinedMode(Object *gui, BOOL combinedMode)
{
	JoystickView* v = (JoystickView*)gui;
	v->CHK_Combined->SetValue(combinedMode);
}

void GUI_SetJoystickAnalogMode(Object *gui, ULONG joystickId, BOOL analogMode)
{
	JoystickView* v = (JoystickView*)gui;
	if (joystickId == 0) {
		v->CHK_Joy0->SetValue(analogMode);
	} else {
		v->CHK_Joy1->SetValue(analogMode);
	}
}
